// ============================================================================
// $Id: tokenizer.cc,v 1.8 1998/03/20 15:56:10 chris Exp $
//
// This is part of Realm Of Magic
//
// Usage: Member functions for class tokenizer
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1993-1996 by the Hackerz of the Realm Of Magic
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// ============================================================================

#include "tokenizer.hh"

tokenizer::tokenizer()
{
  emptystate = true;  
}

tokenizer::tokenizer(const string tokstr, const string sepstring)
{
  string addtoken;
  string::size_type pos1;
  string::size_type pos2;
  
  emptystate = false;
  tokenizesuccess = false;
  tokenstring = tokstr;
  seperatorstring = sepstring;
  tokenamount = 0;
  
  if (!tokenstring.empty() && !seperatorstring.empty() &&
      !(seperatorstring.size() > tokenstring.size())) {
      
    pos1 = 0;
  
    do {
      pos2 = tokenstring.find(seperatorstring, pos1);
      if (pos2 != pos1) {
        tokenizesuccess = true;
        if (pos2 != string::npos)
          addtoken = tokenstring.substr(pos1, pos2 - pos1);
        else
          addtoken = tokenstring.substr(pos1, tokenstring.size() - pos1);
        if (!addtoken.empty()) {
          tokens.push_back(addtoken);
          ++tokenamount;
        }
      }
      pos1 = pos2 + seperatorstring.size();
      if (pos1 >= tokenstring.size())
        break;
    
    } while (pos2 != string::npos);
  }
}

tokenizer::tokenizer(tokenizer &orgtok)
{
  tokenstring = orgtok.tokenstring;
  seperatorstring = orgtok.seperatorstring;

  emptystate = false;
 
  copy(orgtok.tokens.begin(), orgtok.tokens.end(),
       back_inserter(tokens));
  tokenamount = orgtok.tokenamount;
  tokenizesuccess = orgtok.tokenizesuccess;
}

tokenizer::tokenizer(const string tokstr, const string sepstring,
  string::size_type idx_start) {
 
  string addtoken;
  string::size_type pos1;
  string::size_type pos2;

  emptystate = false;
  tokenizesuccess = false;
  tokenstring = tokstr.substr(idx_start);
  seperatorstring = sepstring;
  tokenamount = 0;

  if (!tokenstring.empty() && !seperatorstring.empty() &&
      !(seperatorstring.size() > tokenstring.size())) {

    pos1 = 0;

    do {
      pos2 = tokenstring.find(seperatorstring, pos1);
      if (pos2 != pos1) {
        tokenizesuccess = true;
        if (pos2 != string::npos)
          addtoken = tokenstring.substr(pos1, pos2 - pos1);
        else
          addtoken = tokenstring.substr(pos1, tokenstring.size() - pos1);
        if (!addtoken.empty()) {
          tokens.push_back(addtoken);
          ++tokenamount;
        }
      }
      pos1 = pos2 + seperatorstring.size();
      if (pos1 >= tokenstring.size())
        break;

    } while (pos2 != string::npos);
  }
}

const char * tokenizer::operator[] (int i)
{
  if (emptystate) return 0;
  if (!tokenizesuccess) return 0;
  if ((i < 0) || ((i + 1) > tokenamount)) return 0;
  
  list<string>::iterator lpos;
  
  int j = 0;
  for (lpos = tokens.begin(); lpos != tokens.end(); ++lpos) {
    if (j == i) break;
    ++j;
  }
  return (*lpos).c_str();
}

void tokenizer::clear()
{
  emptystate = true;
  tokenizesuccess = false;
  tokenstring = "";
  tokens.erase(tokens.begin(), tokens.end());
  seperatorstring = "";
  tokenamount = 0;
}

bool tokenizer::tokenize(string tkstr, string spstr)
{
  string addtoken;
  string::size_type pos1;
  string::size_type pos2;
  
  if (!emptystate) return false;
  
  emptystate = false;
  tokenizesuccess = false;
  tokenstring = tkstr;
  seperatorstring = spstr;
  tokenamount = 0;

  if (tokenstring.empty() || seperatorstring.empty() ||
      (seperatorstring.size() > tokenstring.size()))
    return false;

  pos1 = 0;

  do {
    pos2 = tokenstring.find(seperatorstring, pos1);
    if (pos2 != pos1) {
      tokenizesuccess = true;
      if (pos2 != string::npos)
        addtoken = tokenstring.substr(pos1, pos2 - pos1);
      else
        addtoken = tokenstring.substr(pos1, tokenstring.size() - pos1);
      if (!addtoken.empty()) {
        tokens.push_back(addtoken);
        ++tokenamount;
      }
    }
    pos1 = pos2 + seperatorstring.size();
    if (pos1 >= tokenstring.size())
      break;

  } while (pos2 != string::npos);
  return true;
}

